package polyglot.ext.ibex.lr;

import polyglot.util.FilteringIterator;
import polyglot.util.Predicate;
import java.util.*;
import polyglot.main.Report;

class RootedGraph implements Graph
{
    Graph srGraph;
    Node root;
    BitSet reachable;

    RootedGraph(Node root, Graph srGraph) {
        this.root = root;
        this.srGraph = srGraph;

        // Compute the nodes reachable from the root.
        reachable = new BitSet();
        reachable.set(root().index());

        for (Iterator i = srGraph.succs(root()); i.hasNext(); ) {
            Edge e = (Edge) i.next();
            computeReachableDFS(e.target());
        }

        if (Report.should_report(TOPICS, 4)) {
            Report.report(4, "reachable = " + reachable);
        }
    }

    public Iterator nodes() {
        return new FilteringIterator(srGraph.nodes(),
            new Predicate() {
                public boolean isTrue(Object o) {
                    return o instanceof Node &&
                           reachableFromRoot((Node) o);
                }
            });
    }

    private void computeReachableDFS(Node n) {
        if (reachable.get(n.index())) {
            return;
        }

        reachable.set(n.index());

        for (Iterator i = srGraph.succs(n); i.hasNext(); ) {
            Edge e = (Edge) i.next();
            computeReachableDFS(e.target());
        }
    }

    public int size() {
        return srGraph.size();
    }

    public Node root() {
        return root;
    }

    public boolean reachableFromRoot(Node n) {
        return reachable.get(n.index());
    }

    public Iterator succs(Node n) {
        if (n != root() && ! reachableFromRoot(n)) {
            return Collections.EMPTY_LIST.iterator();
        }

        return new FilteringIterator(srGraph.succs(n),
            new Predicate() {
                public boolean isTrue(Object o) {
                    return o instanceof Edge &&
                           reachableFromRoot(((Edge) o).target());
                }
            });
    }

    public Iterator preds(Node n) {
        if (! reachableFromRoot(n)) {
            return Collections.EMPTY_LIST.iterator();
        }

        return new FilteringIterator(srGraph.preds(n),
            new Predicate() {
                public boolean isTrue(Object o) {
                    return o instanceof Edge &&
                           reachableFromRoot(((Edge) o).target());
                }
            });
    }

    static Collection TOPICS = Arrays.asList(
                new String[] { "lr", "ibex", "srgraph" });
}
