package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import java.util.*;

/**
 * An <code>AmbNewArray</code> is an ambiguous piece of syntax
 * that represents the creation of a new array, where the
 * parser is unable to distinguish whether the first dimension
 * in the creation is a parametric instantiation or an expression
 * indicating size.  So a bit of semantic analysis (during
 * the disambiguation passes of the compiler) is necessary
 * to determine what the node should actually be.
 * 
 * For example, in new A[B][C], A must be a type and C must be
 * an expression.  B, however, could be either.
 * 
 * @see polyglot.ext.polyj.parse.InstOrAccess
 */ 

public interface AmbNewArray extends NewArray, Ambiguous
{
    /**
     * Return "B" from "new A[B][C]".
     */
    Receiver param();
    AmbNewArray param(Receiver param);
}
