package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import polyglot.ext.jl.ast.*;

import java.util.*;

public class ClassConstraint_c extends Node_c implements ClassConstraint {
    /** 
     * name of the parameter which is constrained by this
     *  where-clause
     */
    protected String name;

    protected List constraints;

    public ClassConstraint_c(Position pos, String name, List constraints)
    {
        super(pos);
        this.name = name;
        this.constraints = TypedList.copyAndCheck(constraints,
                                                  ProcedureConstraint.class, true);
    }

    protected ClassConstraint reconstruct(List constraints)
    {
        if (CollectionUtil.equals(this.constraints, constraints)) {
            return this;
        } else {
            ClassConstraint_c n = (ClassConstraint_c) copy();
            n.constraints = TypedList.copyAndCheck(constraints,
                                                   ProcedureConstraint.class, true);
            return n;
        }
    }

    public String name() {
        return name;
    }

    public List constraints() {
        return constraints;
    }

    public Node visitChildren(NodeVisitor v) 
    {
        List newConstraints = visitList(constraints, v);
        return reconstruct(newConstraints);
    }

    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        w.write("where ");
        w.write(name);
        w.write(" ");

        if (constraints.isEmpty()) {
            w.write("{ }");
        }
        else {
            w.write("{");
            w.allowBreak(4, " ");
            w.begin(0);

            for (Iterator i = constraints.iterator(); i.hasNext();) {
                ProcedureConstraint e = (ProcedureConstraint) i.next();

                printBlock(e, w, tr);

                if (i.hasNext()) {
                    w.allowBreak(0, " ");
                }
            }

            w.end();
            w.allowBreak(0, " ");
            w.write("}");
        }
    }

    /** Do nothing to translate. */
    public void translate(CodeWriter w, Translator tr) { }

    public void dump(CodeWriter w) {
        w.write("CLASS CONSTRAINT");
        w.write(" NAME:< ");
        w.write(name);
        w.write(" > CONSTRAINTS:< ");
        for (Iterator iter = constraints.iterator(); iter.hasNext();) {
            ((ProcedureConstraint) iter.next()).dump(w);
        }
        w.write(" >");
    }
}
