package polyglot.ext.polyj.types.proxy;

/**
 * Numeric is the base class for all the primitive numeric (byte, char,
 * short, int, long, float, double) proxy types.
 */
public abstract class Numeric {
    public abstract Object clone();

    public abstract byte byteValue();
    public abstract char charValue();
    public abstract short shortValue();
    public abstract int intValue();
    public abstract long longValue();
    public abstract float floatValue();
    public abstract double doubleValue();

    public abstract int signum();
    public abstract boolean equals(long l);
    public abstract boolean equals(double d);
    public abstract boolean less(Numeric n);
    public abstract boolean greater(Numeric n);
    public abstract int compareTo(Numeric n);

    /*
     * These methods should be implemented by subclasses
     * to represent the appropriate Java typing rules.

    public abstract Numeric negate();
    public abstract Numeric abs();
    public abstract Numeric add(Numeric n);
    public abstract Numeric subtract(Numeric n);
    public abstract Numeric multiply(Numeric n);
    public abstract Numeric divide(Numeric n);
    public abstract Numeric mod(Numeric i);
    public abstract Numeric min(Numeric n);
    public abstract Numeric max(Numeric n);
    public abstract Numeric not();
    public abstract Numeric and(Numeric b);
    public abstract Numeric or(Numeric b);
    public abstract Numeric xor(Numeric b);

    */

}
